"use strict";

module.exports = [
	[
		/export '__esModule' \(imported as 'm1'\) was not found in '\.\/analyzable-module\.js'/
	],
	[/export 'default' \(imported as 'm3'\) was not found in '\.\/esModule\.js'/],
	[/export 'default' \(imported as 'd3'\) was not found in '\.\/esModule\.js'/],
	[
		/export '__esModule' \(imported as 'm1'\) was not found in '\.\/analyzable-module\.js'/
	],
	[/export 'default' \(imported as 'm3'\) was not found in '\.\/esModule\.js'/],
	[/export 'default' \(imported as 'd3'\) was not found in '\.\/esModule\.js'/]
];
