import { a as concat2_a, b as concat2_b, c as concat2_c } from "./concat2";

export const a = "concat1~" + concat2_a;
export const b = "concat1~" + concat2_b;
export const c = "concat1~" + concat2_c;
