"use strict";

module.exports = [
	[
		/export '__esModule' \(imported as 'm1'\) was not found in '\.\/analyzable-module\.js'/
	],
	[
		/export '__esModule' \(imported as 'm1'\) was not found in '\.\/analyzable-module\.js'/
	]
];
