/*
 * This file was automatically generated.
 * DO NOT MODIFY BY HAND.
 * Run `yarn fix:special` to update
 */
"use strict";module.exports=r,module.exports.default=r;const e={type:"object",additionalProperties:!1,properties:{esModule:{$ref:"#/definitions/CssGeneratorEsModule"},exportType:{$ref:"#/definitions/CssParserExportType"},exportsConvention:{$ref:"#/definitions/CssGeneratorExportsConvention"},exportsOnly:{$ref:"#/definitions/CssGeneratorExportsOnly"},localIdentHashDigest:{$ref:"#/definitions/HashDigest"},localIdentHashDigestLength:{$ref:"#/definitions/HashDigestLength"},localIdentHashFunction:{$ref:"#/definitions/HashFunction"},localIdentHashSalt:{$ref:"#/definitions/HashSalt"},localIdentName:{$ref:"#/definitions/CssGeneratorLocalIdentName"}}},t=Object.prototype.hasOwnProperty;function n(r,{instancePath:s="",parentData:o,parentDataProperty:a,rootData:i=r}={}){let l=null,p=0;if(0===p){if(!r||"object"!=typeof r||Array.isArray(r))return n.errors=[{params:{type:"object"}}],!1;{const s=p;for(const s in r)if(!t.call(e.properties,s))return n.errors=[{params:{additionalProperty:s}}],!1;if(s===p){if(void 0!==r.esModule){const e=p;if("boolean"!=typeof r.esModule)return n.errors=[{params:{type:"boolean"}}],!1;var f=e===p}else f=!0;if(f){if(void 0!==r.exportType){let e=r.exportType;const t=p;if("link"!==e&&"text"!==e&&"css-style-sheet"!==e&&"style"!==e)return n.errors=[{params:{}}],!1;f=t===p}else f=!0;if(f){if(void 0!==r.exportsConvention){let e=r.exportsConvention;const t=p,s=p;let o=!1;const a=p;if("as-is"!==e&&"camel-case"!==e&&"camel-case-only"!==e&&"dashes"!==e&&"dashes-only"!==e){const e={params:{}};null===l?l=[e]:l.push(e),p++}var c=a===p;if(o=o||c,!o){const t=p;if(!(e instanceof Function)){const e={params:{}};null===l?l=[e]:l.push(e),p++}c=t===p,o=o||c}if(!o){const e={params:{}};return null===l?l=[e]:l.push(e),p++,n.errors=l,!1}p=s,null!==l&&(s?l.length=s:l=null),f=t===p}else f=!0;if(f){if(void 0!==r.exportsOnly){const e=p;if("boolean"!=typeof r.exportsOnly)return n.errors=[{params:{type:"boolean"}}],!1;f=e===p}else f=!0;if(f){if(void 0!==r.localIdentHashDigest){const e=p;if("string"!=typeof r.localIdentHashDigest)return n.errors=[{params:{type:"string"}}],!1;f=e===p}else f=!0;if(f){if(void 0!==r.localIdentHashDigestLength){let e=r.localIdentHashDigestLength;const t=p;if(p==p){if("number"!=typeof e)return n.errors=[{params:{type:"number"}}],!1;if(e<1||isNaN(e))return n.errors=[{params:{comparison:">=",limit:1}}],!1}f=t===p}else f=!0;if(f){if(void 0!==r.localIdentHashFunction){let e=r.localIdentHashFunction;const t=p,s=p;let o=!1;const a=p;if(p===a)if("string"==typeof e){if(e.length<1){const e={params:{}};null===l?l=[e]:l.push(e),p++}}else{const e={params:{type:"string"}};null===l?l=[e]:l.push(e),p++}var u=a===p;if(o=o||u,!o){const t=p;if(!(e instanceof Function)){const e={params:{}};null===l?l=[e]:l.push(e),p++}u=t===p,o=o||u}if(!o){const e={params:{}};return null===l?l=[e]:l.push(e),p++,n.errors=l,!1}p=s,null!==l&&(s?l.length=s:l=null),f=t===p}else f=!0;if(f){if(void 0!==r.localIdentHashSalt){let e=r.localIdentHashSalt;const t=p;if(p==p){if("string"!=typeof e)return n.errors=[{params:{type:"string"}}],!1;if(e.length<1)return n.errors=[{params:{}}],!1}f=t===p}else f=!0;if(f)if(void 0!==r.localIdentName){let e=r.localIdentName;const t=p,s=p;let o=!1;const a=p;if("string"!=typeof e){const e={params:{type:"string"}};null===l?l=[e]:l.push(e),p++}var d=a===p;if(o=o||d,!o){const t=p;if(!(e instanceof Function)){const e={params:{}};null===l?l=[e]:l.push(e),p++}d=t===p,o=o||d}if(!o){const e={params:{}};return null===l?l=[e]:l.push(e),p++,n.errors=l,!1}p=s,null!==l&&(s?l.length=s:l=null),f=t===p}else f=!0}}}}}}}}}}return n.errors=l,0===p}function r(e,{instancePath:t="",parentData:s,parentDataProperty:o,rootData:a=e}={}){let i=null,l=0;return n(e,{instancePath:t,parentData:s,parentDataProperty:o,rootData:a})||(i=null===i?n.errors:i.concat(n.errors),l=i.length),r.errors=i,0===l}