export * as NS from "./index.js";

var module = "should not conflict",
	define = "should not conflict",
	require = "should not conflict",
	exports = "should not conflict",
	globalName = "should not conflict";
console.log.bind(console, module, define, require, exports, globalName);
