export * as ca from "./a";
export * as cb from "./b";
export const caCanMangle = __webpack_exports_info__.ca.canMangle;
export const cbCanMangle = __webpack_exports_info__.cb.canMangle;
export const ca_aaaCanMangle = __webpack_exports_info__.ca.aaa.canMangle;
export const cb_bbbCanMangle = __webpack_exports_info__.cb.bbb.canMangle;