@import url("./foo.css");
@import url("./auto-text-no-esm.css");

@import url("./imported-layer.css") layer(utilities);
@import url("./imported-base.css") supports(display: grid);
@import url("./imported-base.css") screen and (min-width: 768px);
@import url("./imported-layer.css") layer(components) supports(display: flex) screen and (max-width: 1024px);

.main-class {
	color: purple;
	display: flex;
	background-image: url("./img.png");
}

@layer base {
	.base-class {
		font-family: sans-serif;
	}
}

@supports (display: grid) {
	.grid-class {
		display: grid;
	}
}

@media (min-width: 600px) {
	.responsive-class {
		width: 100%;
	}
}

