exports.abc = "abc";
exports[""] = "";
exports["default"] = "default";
exports["0"] = "0";
exports[1] = 1;
exports.length = "length";
exports["0_0"] = "0_0";
exports.if = "if";
exports["\0"] = "\0";
exports["\n"] = "\n";
exports["*/"] = "*/";
exports["a.b.c"] = "a.b.c";
