"use strict";

module.exports = [
	[
		/export 'default' \(imported as 'def'\) was not found in '\.\/module' \(possible exports: b, c, d\)/
	],
	[
		/export 'a' \(imported as 'aa'\) was not found in '\.\/module' \(possible exports: b, c, d\)/
	],
	[
		/export 'e' \(imported as 'e'\) was not found in '\.\/module' \(possible exports: b, c, d\)/
	]
];
