"use strict";

module.exports = [
	[/"global" has been used, it will be undefined in next major version/],
	[/"__filename" has been used, it will be undefined in next major version/],
	[/"__dirname" has been used, it will be undefined in next major version/],

	[/"global" has been used, it will be undefined in next major version/],

	[/"__filename" has been used, it will be undefined in next major version/],
	[/"__dirname" has been used, it will be undefined in next major version/]
];
