Object.defineProperty(exports, "__esModule", { value: true });
exports.main = main;
function main() {}

// commonjs bailout
if(this){}

it("Should work when exports are unprovided", function() {
	expect(true).toBe(true);
});
