"use strict";

module.exports = [
	{
		code: /DEP_WEBPACK_RULE_LOADER_OPTIONS_STRING/,
		message:
			/Using a string as loader options is deprecated \(ruleSet\[1\]\.rules\[2\]\.options\)/
	},
	{
		code: /DEP_WEBPACK_RULE_LOADER_OPTIONS_STRING/,
		message:
			/Using a string as loader options is deprecated \(ruleSet\[1\]\.rules\[3\]\.use\[0\]\.options\)/
	}
];
