import * as basic from "./basic.module.css";
import * as styles from "./classes.module.css";
import * as styles1 from "./composes-multiple.module.css";
import * as styles3 from "./composes-global.module.css";
import * as styles4 from "./scope-at-rule.module.css";
import * as styles5 from "./nesting.module.css";
import * as styles6 from "./prefer-relative.module.css";
import * as styles7 from "./animation-name.module.css";
import * as styles8 from "./at-sign-in-package-name.module.css";
import * as styles9 from "./resolving-from-node_modules.module.css";
import * as styles10 from "./local-Ident-name.module.css";
import * as styles11 from "./local-Ident-name.module.css?local-ident-name-1";
import * as styles12 from "./local-Ident-name.module.css?local-ident-name-2";
import * as styles13 from "./local-Ident-name.module.css?local-ident-name-3";
import * as styles14 from "./local-Ident-name.module.css?local-ident-name-4";
import * as styles15 from "./local-Ident-name.module.css?local-ident-name-5";
import * as styles16 from "./local-Ident-name.module.css?local-ident-name-6";
import * as styles17 from "./local-Ident-name.module.css?local-ident-name-7";
import * as styles18 from "./local-Ident-name.module.css?local-ident-name-8";
import * as styles19 from "./local-Ident-name.module.css?local-ident-name-9";
import * as styles20 from "./order.module.css";
import * as styles21 from "./dedup.module.css";
import * as styles22 from "./composes-from-less.module.css";
import * as styles23 from "./tilde.module.css";
import * as styles24 from "./icss.module.css";

it("should work", () => {
	const links = document.getElementsByTagName("link");
	const css = [];

	// Skip first because import it by default
	for (const link of links.slice(1)) {
		css.push(link.sheet.css);
	}

	expect(basic).toMatchSnapshot();
	expect(styles).toMatchSnapshot();
	expect(styles1).toMatchSnapshot();
	expect(styles3).toMatchSnapshot();
	expect(styles4).toMatchSnapshot();
	expect(styles5).toMatchSnapshot();
	expect(styles6).toMatchSnapshot();
	expect(styles7).toMatchSnapshot();
	expect(styles8).toMatchSnapshot();
	expect(styles9).toMatchSnapshot();
	expect(styles10).toMatchSnapshot();
	expect(styles11).toMatchSnapshot();
	expect(styles12).toMatchSnapshot();
	expect(styles13).toMatchSnapshot();
	expect(styles14).toMatchSnapshot();
	expect(styles15).toMatchSnapshot();
	expect(styles16).toMatchSnapshot();
	expect(styles17).toMatchSnapshot();
	expect(styles18).toMatchSnapshot();
	expect(styles19).toMatchSnapshot();
	expect(styles20).toMatchSnapshot();
	expect(styles21).toMatchSnapshot();
	expect(styles22).toMatchSnapshot();
	expect(styles23).toMatchSnapshot();
	expect(styles24).toMatchSnapshot();
	expect(css).toMatchSnapshot();
});
