"use strict";

module.exports = [
	[/Critical dependency/, { moduleName: /templateLoader\.js/ }],
	[/Critical dependency/, { moduleName: /templateLoaderIndirect\.js/ }],
	[/Critical dependency/, { moduleName: /templateLoaderIndirect\.js/ }]
];
