"use strict";

module.exports = [
	[/export 'global' \(imported as 'style'\) was not found/],
	[/export 'nested2' \(imported as 'style'\) was not found/],
	[/export 'global-color' \(imported as 'style'\) was not found/],
	[/export 'GLOBAL-COLOR' \(imported as 'style'\) was not found/],
	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],
	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/],

	[/export 'global' \(imported as 'style'\) was not found/],
	[/export 'nested2' \(imported as 'style'\) was not found/],
	[/export 'global-color' \(imported as 'style'\) was not found/],
	[/export 'GLOBAL-COLOR' \(imported as 'style'\) was not found/],
	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],
	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/],

	[/export 'global' \(imported as 'style'\) was not found/],
	[/export 'nested2' \(imported as 'style'\) was not found/],
	[/export 'global-color' \(imported as 'style'\) was not found/],
	[/export 'GLOBAL-COLOR' \(imported as 'style'\) was not found/],
	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],
	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/],

	[/export 'global' \(imported as 'style'\) was not found/],
	[/export 'nested2' \(imported as 'style'\) was not found/],
	[/export 'global-color' \(imported as 'style'\) was not found/],
	[/export 'GLOBAL-COLOR' \(imported as 'style'\) was not found/],
	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],
	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/],

	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/],
	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],

	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/],
	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],

	[/Missing whitespace after ':global' in ':global\.class-no-space \{/],
	[
		/Missing whitespace after ':global' in ':global\/\*\* test \*\*\/\.class \{/
	],
	[/Missing whitespace after ':local' in ':local\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\.class \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/#hash \{'/],
	[/Missing whitespace after ':local' in ':local\/\*\* test \*\*\/\{/],
	[/Broken '@value' at-rule: @value;'/],
	[/Broken '@value' at-rule: @value test;'/]
];
