__webpack_hash__;

//@ts-expect-error
window.__webpack_hash__;

__webpack_require__(1);
//@ts-expect-error
global.__webpack_require__(1);

__webpack_require__("1");
//@ts-expect-error
global.__webpack_require__("1");

if (module.hot) {
}

__webpack_exports_info__.exportA.used;
__webpack_exports_info__.exportA.b.c.provideInfo;
