"use strict";

module.exports = [
	[
		/Invalid generator object\. Asset Modules Plugin has been initialized using a generator object that does not match the API schema/,
		/generator has an unknown property 'filename'/
	]
];
