"use strict";

module.exports = [
	/Can't import the named export 'aa' \(imported as 'aa'\) from default-exporting module/,
	/Can't import the named export 'named' \(imported as 'named'\) from default-exporting module/
];
