"use strict";

module.exports = [
	/javascript\/auto error message/,
	/asset\/inline error message/,
	/asset\/resource error message/,
	/asset\/resource other error message/,
	/asset\/resource in css error message/,
	/asset\/source error message/,
	/asset\/source in css error message/,
	/css\/auto error message/,
	/css error message/,
	/json error message/,
	/json other error message/,
	/webassembly\/async error message/
];
