.className {
	color: red
}

.otherClassName {
	color: green;
}

.otherClassNameToo {
	color: blue;
}

.from {
	color: yellow;
}

.a {
	color: red;
}

.-b {
	color: red;
}

.b- {
	color: red;
}

.--c {
	color: red;
}

.c-- {
	color: red;
}

._d {
	color: red;
}

.d\% {
	color: tan;
}
