"use strict";

module.exports = [
	[/used with `import \* as namespace from '...'`/],
	[/used with `import \* as namespace from '...'`/],
	[/used with `import \* as namespace from '...'`/],
	[/is not a part of the Import Defer proposal/],
	[/is not a part of the Import Defer proposal/]
];
