/* globals it */
"use strict";

it("should throw validation error when validate is true and IgnorePlugin options are invalid", function() {
	// This test case is handled by the errors.js file
	// The validation error should be caught during compilation
	expect(true).toBe(true);
});
