exports.property1 = require("./module?pe1" + __resourceQuery).abc;
var m2 = require("./module?pe2" + __resourceQuery);
exports.property2 = m2.abc;
this.property3 = require("./module?pe3" + __resourceQuery).abc;
var m4 = require("./module?pe4" + __resourceQuery);
this.property4 = m4.abc;
