:root {
	--main-color: #06c;
	--accent-color: #006;
}

.foo {
	--fg-color: blue;
}

#foo h1 {
	color: var(--main-color);
}

@--custom {}
@--library1-custom {}

.class {
	--vendor-property: --vendor-function("test");
}
