"use strict";

module.exports = [
	/Conflicting @charset at-rules detected: the module css \.\/styles-3\.link\.css \(exportType: link\) \(in chunk main\) specifies "UNKNOWN", but "UTF-8" was expected, all modules must use the same character set/
];
