@value red-v1 blue;
@value red-i: blue;
@value blue-v1 red;
@value blue-i: red;
@value a: "test-a";
@value b: "test-b";
@value --red: var(--color);
@value test-v1: blue;
@value test-v2: blue;
@value red-v2: blue;
@value green-v2: yellow;
@value red-v3: blue;
@value red-v4: blue;
