@value primary from "./theme.module.css";
@value error from "./theme.module.css";

@value colors: "./theme.module.css";
@value danger: error from colors;

.color {
	color: danger
}

.button {
	background-color: primary;
	color: white;
}

.button-error {
	background-color: error;
	border: 2px solid darkred;
}
