/*
 * This file was automatically generated.
 * DO NOT MODIFY BY HAND.
 * Run `yarn fix:special` to update
 */
"use strict";function o(s,{instancePath:r="",parentData:m,parentDataProperty:t,rootData:e=s}={}){return"var"!==s&&"module"!==s&&"assign"!==s&&"this"!==s&&"window"!==s&&"self"!==s&&"global"!==s&&"commonjs"!==s&&"commonjs2"!==s&&"commonjs-module"!==s&&"commonjs-static"!==s&&"amd"!==s&&"amd-require"!==s&&"umd"!==s&&"umd2"!==s&&"jsonp"!==s&&"system"!==s&&"promise"!==s&&"import"!==s&&"module-import"!==s&&"script"!==s&&"node-commonjs"!==s&&"asset"!==s&&"css-import"!==s&&"css-url"!==s?(o.errors=[{params:{}}],!1):(o.errors=null,!0)}module.exports=o,module.exports.default=o;