"use strict";

module.exports = {
	findBundle() {
		return [
			"pseudo-export_style_module_css.bundle0.js",
			"pseudo-export_style_module_css_module.bundle0.js",
			"pseudo-export_style_module_css_exportsOnly.bundle0.js",
			"bundle0.js"
		];
	}
};
