"use strict";

module.exports = [
	/Critical dependency: the request of a dependency is an expression/,
	/Critical dependency: the request of a dependency is an expression/,
	/Critical dependency: the request of a dependency is an expression/,
	/Critical dependency: the request of a dependency is an expression/
];
