"use strict";

module.exports = [
	[
		/\.\/loader-1\.js/,
		/Loader has been/,
		/options\.arg6\.bar\.baz should be a string/
	],
	[
		/\.\/loader-2\.js/,
		/Custom Loader Name has been/,
		/configuration\.arg should be true/
	]
];
