"use strict";

module.exports = {
	findBundle() {
		return [
			"shared.js",
			"a.js",
			"b.js",
			"c1.js",
			"c2.js",
			"ax.js",
			"bx.js",
			"cx1.js",
			"cx2.js",
			"d1.js",
			"d2.js"
		];
	}
};
