export const aaa = 1;
export const bbb = 2;
export const object = {};
export * as ddd from "./n";
export const usedA = __webpack_exports_info__.aaa.used;
export const canMangleA = __webpack_exports_info__.ccc.canMangle;
export const usedB = __webpack_exports_info__.bbb.used;
