"use strict";

module.exports = [
	/Any '@import' rules must precede all other rules/,
	/Any '@import' rules must precede all other rules/,
	/Any '@import' rules must precede all other rules/,
	/Any '@import' rules must precede all other rules/
];
