"use strict";

module.exports = [
	[
		/UnhandledSchemeError: Reading from "https:\/\/test\.cases\/url-external\.css"/
	],
	[/UnhandledSchemeError: Reading from "https:\/\/test\.cases\/external\.css"/],
	[
		/UnhandledSchemeError: Reading from "https:\/\/test\.cases\/url-external\.css"/
	],
	[/UnhandledSchemeError: Reading from "https:\/\/test\.cases\/external\.css"/]
];
