export const a = () => "a";
export const A = "A";
export const b = "b";
export const B = "B";
export const c = "c";
export const C = "C";
export const d = () => "d";
export const D = "D";
export const e = "e";
export const E = "E";
export const f = "f";
export const F = "F";
export class X {
	x = "X";
}
export const y = "y";
