"use strict";

module.exports = [
	[
		/Should not import the named export 'a' \(reexported as 'a'\) from default-exporting module \(only default export is available soon\)/
	],
	[
		/Should not import the named export 'b' \(reexported as 'b'\) from default-exporting module \(only default export is available soon\)/
	]
];
