export class A3 {}
export class B3 {}
export class C3 {}
export class D3 {}
export class E3 {}
export class F3 {}
export class Pure3 {}
export class ConditionalExpression3 extends (true ? A3 : B3) {}
export class LogicalExpression3 extends (A3 || B3) {}
