a {
	color: #000000;
	color: #ffffff;
	color: #FFFFFF;
	color: #0000ffcc;
	color: #0000FFCC;
	color: #000;
	color: #fff;
	color: #FFF;
	color: #0000;
	color: #ffff;
	color: #FFFF;
	color: #1;
	color: #FF;
	color: #123456789;
	color: #abc;
	color: #aa\61;
}
