import { a as external_a, b as external_b, c as external_c } from "external";

export const a = "concat2~" + external_a;
export const b = "concat2~" + external_b;
export const c = "concat2~" + external_c;
