"use strict";

module.exports = [
	/Compilation error while processing magic comment\(-s\): \/\*\*\*\*webpackIgnore: false\*\*\*\//,
	/Compilation error while processing magic comment\(-s\): \/\* {3}\* {3}\* {3}\* {3}webpackIgnore: {3}false {3}\* {3}\* {3}\*\//,
	/`webpackIgnore` expected a boolean, but received: 1\./,
	/`webpackIgnore` expected a boolean, but received: 1\./,
	/`webpackIgnore` expected a boolean, but received: 1\./,
	/`webpackIgnore` expected a boolean, but received: 1\./,
	/Compilation error while processing magic comment\(-s\): \/\*\*\*webpackIgnore: {2}true\*\*\*\//,
	/Compilation error while processing magic comment\(-s\): \/\*\*\*webpackIgnore: {2}true\*\*\*\//,
	/Compilation error while processing magic comment\(-s\): \/\*\*\*webpackIgnore: {2}true\*\*\*\//,
	/Compilation error while processing magic comment\(-s\): \/\*\*\*webpackIgnore: {2}true\*\*\*\//
];
