import { b } from "./b";

export function a() {
	return b();
}
export const obj = {};
export const aUsed = __webpack_exports_info__.a.used;
export const aProvided = __webpack_exports_info__.a.provideInfo;
export const objUsed = __webpack_exports_info__.obj.used;
export const objAProvided = __webpack_exports_info__.obj.A.provideInfo;
export const aToStringProvided = __webpack_exports_info__.a.toString.provideInfo;
