export * as named1 from "./a.js";
export { named1 as named2 } from "./a.js";
export { named1 as named3 } from "./a.js";
export * as named4 from "./a.js";
export { named5 } from "./a.js";
export { named1 as named6 } from "./a.js";
