"use strict";

module.exports = [
	/"__dirname" has been used/,
	/"__filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"__dirname" has been used/,
	/"__filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"__dirname" has been used/,
	/"__filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"__dirname" has been used/,
	/"__filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"__dirname" has been used/,
	/"__filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/,
	/"import.meta.dirname" has been used/,
	/"import.meta.filename" has been used/
];
