import * as dep from /* webpackDefer: true */ "./reexport-async-dep.js";

__configCases__deferImport__proposal.push("START reexport-async.js");

export { dep };

__configCases__deferImport__proposal.push("END reexport-async.js");
