"use strict";

module.exports = {
	moduleScope(scope) {
		// const link = scope.window.document.createElement("link");
		// link.rel = "stylesheet";
		// link.href = "style_css.css";
		// scope.window.document.head.appendChild(link);
	}
};
