import a from "./file-a1";
import b from "./file-b1";
import A from "./file-A2";
import B from "./file-B2";
export default "ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
	a +
	b +
	A +
	B;
