export let counter = 0;
export const d = 1;
export const c = 1;
export const e = 1;
export const f = 1;
export const g = 1;

export const exportsInfo = {
	counter: __webpack_exports_info__.counter.used,
	d: __webpack_exports_info__.d.used,
	c: __webpack_exports_info__.c.used,
	e: __webpack_exports_info__.e.used,
	f: __webpack_exports_info__.f.used,
	g: __webpack_exports_info__.g.used
};
