"use strict";

module.exports = [
	[
		/Should not import the named export '2' \(imported as 'c'\) from default-exporting module \(only default export is available soon\)/
	],
	[
		/Should not import the named export 'aa' \(imported as 'aa'\) from default-exporting module \(only default export is available soon\)/
	],
	[
		/Should not import the named export 'bb' \(imported as 'bb'\) from default-exporting module \(only default export is available soon\)/
	],
	[
		/Should not import the named export 'named' \(imported as 'named'\) from default-exporting module \(only default export is available soon\)/
	],
	[
		/Should not import the named export 'named' \(imported as 'gnamed'\) from default-exporting module \(only default export is available soon\)/
	]
];
