export { default as number } from "./tla";
export { getNumber } from "./wasm.wat"
export { default as moduleRequest } from "external-module"
export { default as importRequest } from "external-import"
export { default as promiseRequest } from "external-promise"
export { default as scriptRequest } from "external-script"
