"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.greet = void 0;
var greet = function (name) {
    if (typeof name !== "string") {
        throw new TypeError("Invalid name type");
    }
    return "Hello, ".concat(name, "!");
};
exports.greet = greet;
//# sourceMappingURL=index.js.map