"use strict";

module.exports = [
	/Composition is only allowed when selector is single local class name not in "exportName19", "exportName19-nested", "exportName19-nested-nested"/,
	/Incorrect composition, expected global keyword or string value/,
	/Incorrect composition, expected class named/,
	/Composition is only allowed when selector is single local class name not in "exportName19", "exportName20"/,
	/Composition is only allowed when selector is single local class name not in "exportName22", "exportName23"/,
	/Composition is only allowed when selector is single local class name not in "exportName25", "exportName26"/,
	/Composition is only allowed when selector is single local class name not in "exportName28", "exportName29"/,
	/Composition is only allowed when selector is single local class name not in "exportName33", "exportName34"/,
	/Composition is only allowed when selector is single local class name not in "exportName37", "exportName38"/,
	/Self-referencing name "exportName40" not found/,
	/Missing whitespace after ':local' in ':local.b \{'/,
	/Missing whitespace after ':global' in ':global, .foobar :global \{'/,
	/Missing whitespace after ':global' in ':global.bar \{'/
];
